#region Header
//
// .NET Sample
//
// Copyright (C) 2005-2007 by Autodesk, Inc.
//
// Permission to use, copy, modify, and distribute this software
// for any purpose and without fee is hereby granted, provided
// that the above copyright notice appears in all copies and
// that both that copyright notice and the limited warranty and
// restricted rights notice below appear in all supporting
// documentation.
//
// AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS.
// AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
// MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC.
// DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
// UNINTERRUPTED OR ERROR FREE.
//
// Use, duplication, or disclosure by the U.S. Government is subject to
// restrictions set forth in FAR 52.227-19 (Commercial Computer
// Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
// (Rights in Technical Data and Computer Software), as applicable.
//
#endregion // Header

#region Namespaces
using System.Reflection;

// AutoCAD
using Autodesk.AutoCAD.Runtime;
using Autodesk.AutoCAD.ApplicationServices;
using Autodesk.AutoCAD.DatabaseServices;
using Autodesk.AutoCAD.EditorInput;
using Autodesk.AutoCAD.Geometry;

// ACA (Aec)
using Autodesk.Aec.DatabaseServices;
using Autodesk.Aec.Arch.DatabaseServices;
using Autodesk.Aec.ApplicationServices;
using Autodesk.Aec.Arch.ApplicationServices;

// To resolve Object/Entity ambiguities
using AcObject = Autodesk.AutoCAD.DatabaseServices.DBObject;
using AecObject = Autodesk.Aec.DatabaseServices.DBObject;
using AcEntity = Autodesk.AutoCAD.DatabaseServices.Entity;
using AecEntity = Autodesk.Aec.DatabaseServices.Entity;

// To resolve Application ambiguities
using AcadApp = Autodesk.AutoCAD.ApplicationServices.Application;

// To resolve TransactionManager ambiguities
using DbTrMgr = Autodesk.AutoCAD.DatabaseServices.TransactionManager; //wraps AcDbTransactionManager
using AppTrMgr = Autodesk.AutoCAD.ApplicationServices.TransactionManager; //wraps AcTransactionManager

// To resolve ObjectId ambiguities
using ObjectId = Autodesk.AutoCAD.DatabaseServices.ObjectId;
using ObjectIdCollection = Autodesk.AutoCAD.DatabaseServices.ObjectIdCollection;

#endregion // Namespaces

namespace AcaNetIntroCs
{
  /// <summary>
  /// A class containing commands.
  /// </summary>
  public class Commands2a
  {
    /// <summary>
    /// Demonstrate DWG Aec and architectural settings.
    /// </summary>
    [CommandMethod( "TestSettings" )]
    static public void TestSettings()
    {
      try
      {
        Database db = HostApplicationServices.WorkingDatabase;
        using( Transaction tr = db.TransactionManager.StartTransaction() )
        {
          //
          // DrawingSetupVariables (in Autodesk.Aec.ApplicationServices)
          // ... wrap most of "Drawing Setup"
          //
          ObjectId idDwgVars = DrawingSetupVariables.GetInstance( db, false );
          DrawingSetupVariables dwgVars = tr.GetObject( idDwgVars, OpenMode.ForWrite ) as DrawingSetupVariables;
          string oldSuffix = dwgVars.AreaSuffix;
          dwgVars.AreaSuffix = "M.sqr.";
          App.MsgBox( "Old Area suffix = " + oldSuffix + "\nNew Area suffix = " + dwgVars.AreaSuffix );
          //
          // ArchDBVariables (in Autodesk.Aec.Arch.ApplicationServices)
          // ... wrap most of "Aec Object Settings" Tab in OPTIONS
          //
          ObjectId idArchVars = ArchDBVariables.GetInstance( db, true );
          AcObject o = tr.GetObject( idArchVars, OpenMode.ForWrite );
          App.MsgBox( "Object type: " + o.GetType().ToString() );
          ArchDBVariables archVars = o as ArchDBVariables;
          //double oldR = archVars.WallCleanupRadius;
          double oldR = archVars.AutosnapSearchRadius;
          //archVars.WallCleanupRadius += 10;
          archVars.AutosnapSearchRadius += 10;
          //string msg = string.Format( "Old Wall Cleanup Radius = {0}\nNew Wall Cleanup Radius = {1}", oldR, archVars.WallCleanupRadius );
          string msg = string.Format("Old Wall auto-snap Radius = {0}\nNew Wall auto-snap Radius = {1}", oldR, archVars.AutosnapSearchRadius);

          App.MsgBox( msg );
          tr.Commit();
        }
      }
      catch( Exception ex )
      {
        App.MsgBox( "Exception: " + ex.Message );
      }
    }

    /// <summary>
    /// Demonstrate adding new Aec entities.
    /// </summary>
    [CommandMethod( "TestWallAdd" )]
    static public void TestWallAdd()
    {
      try
      {
        Editor ed = AcadApp.DocumentManager.MdiActiveDocument.Editor;
        // Get start point - simple GetPoint signature
        PromptPointResult resPtStart = ed.GetPoint( "Start point: " );
        if( PromptStatus.OK != resPtStart.Status ) { return; }
        // Get end point - GetPoint signature using options class to show BasePoint
        PromptPointOptions optPtEnd = new PromptPointOptions( "End point: " );
        optPtEnd.BasePoint = resPtStart.Value;
        optPtEnd.UseBasePoint = true;
        PromptPointResult resPtEnd = ed.GetPoint( optPtEnd );
        if( PromptStatus.OK != resPtStart.Status ) { return; }
        // Start Transaction
        Database db = HostApplicationServices.WorkingDatabase;
        using( Transaction tr = db.TransactionManager.StartTransaction() )
        {
          Wall w = new Wall();
          w.SetDatabaseDefaults( db ); // this WILL also call any OMF-overridden subSetDatabaseDefaults()
          w.SetToStandard( db );
          // Note that StartPoint and some other properties are read-only - must use Set!
          w.Set( resPtStart.Value, resPtEnd.Value, Vector3d.ZAxis );
          // Add to ModelSpace
          BlockTable bt = tr.GetObject( db.BlockTableId, OpenMode.ForRead ) as BlockTable;
          BlockTableRecord btr = tr.GetObject( bt[BlockTableRecord.ModelSpace], OpenMode.ForWrite ) as BlockTableRecord;
          btr.AppendEntity( w );
          // Whenever an object is added to DB, we must also let the transaction know:
          tr.AddNewlyCreatedDBObject( w, true );
          tr.Commit();
        }
      }
      catch( Exception ex )
      {
        App.MsgBox( "Exception: " + ex.Message );
      }
    }

    /// <summary>
    /// Return an English plural suffix 's' or nothing for the given number of items.
    /// </summary>
    static string PluralSuffix( int n )
    {
      return 1 == n ? "" : "s";
    }

    public static string DotOrColon( int n )
    {
      return 0 == n ? "." : ":";
    }

    /// <summary>
    /// Demonstrate style dictionary, e.g. wall styles.
    /// </summary>
    [CommandMethod( "TestWallStyles" )]
    static public void TestWallStyles()
    {
      try
      {
        Database db = HostApplicationServices.WorkingDatabase;
        Editor ed = AcadApp.DocumentManager.MdiActiveDocument.Editor;
        using( Transaction tr = db.TransactionManager.StartTransaction() )
        {
          DictionaryWallStyle dict = new DictionaryWallStyle( db ); // get wall styles dictionary
          // Report existing Styles
          ed.WriteMessage( "\nExisting wall styles: " );
          foreach( string name in dict.NamesInUse )
          {
            ed.WriteMessage( "\n  " + name );
          }
          // Delete purgable styles
          ObjectIdCollection ids = dict.Records;
          // AutoCAD-check (removes any referenced)
          db.Purge( ids );
          // Special AEC check (if dictionary "protects" some non-referenced ids,
          // e.g. see OMF AecDictDispRepConfig or can be overridden for a custom Dict)
          dict.ExtraPurgeCheck( ids );
          int n = ids.Count;
          ed.WriteMessage( string.Format( "\nThere are {0} purgable style{1} to be deleted{2}",
            n, PluralSuffix( n ), DotOrColon( n ) ) );
          // Loop and delete each purgable
          foreach( ObjectId id in ids )
          {
            WallStyle style = tr.GetObject( id, OpenMode.ForWrite ) as WallStyle;
            ed.WriteMessage( "\n  " + style.Name );
            style.Erase();
          }
          tr.Commit();
        }
      }
      catch( Exception ex )
      {
        App.MsgBox( "Exception: " + ex.Message );
      }
    }

    static string TypeAndHandle( AcObject o )
    {
      return o.GetType().Name + " " + o.Handle.ToString();
    }

    /// <summary>
    /// Demonstrate anchors and check anchor type.
    /// </summary>
    [CommandMethod( "TestAnchor", CommandFlags.Modal | CommandFlags.UsePickSet )]
    static public void TestAnchor()
    {
      Database db = HostApplicationServices.WorkingDatabase;
      Editor ed = AcadApp.DocumentManager.MdiActiveDocument.Editor;
      while( true )
      {
        PromptEntityOptions optEnt = new PromptEntityOptions( "Select an ACA entity: " );
        optEnt.SetRejectMessage( "Selected entity is NOT an ACA one, please try again..." );
        optEnt.AddAllowedClass( typeof( Geo ), false );
        PromptEntityResult resEnt = ed.GetEntity( optEnt );
        if( PromptStatus.OK != resEnt.Status ) { return; }
        try
        {
          using( Transaction tr = db.TransactionManager.StartTransaction() )
          {
            // Open the geo (may fail due to the defect - see TCgetWall)
            Geo geo = tr.GetObject( resEnt.ObjectId, OpenMode.ForRead ) as Geo;
            string msg = "Selected " + TypeAndHandle( geo );

            // Check if no anchor
            if( geo.AnchorId.IsNull )
            {
              msg += "\nis NOT anchored!";
            }
            else
            {
              Anchor anc = tr.GetObject( geo.AnchorId, OpenMode.ForRead ) as Anchor;
              msg += "\nis anchored using " + TypeAndHandle( anc );
              // Check if AnchorToReference (all really are, so in pracice could have cast above)
              // Either of the following two will work - one uses ARX hierarchy, the other .NET
              //if( anc.GetRXClass.IsDerivedFrom( RXClass.GetClass( typeof( AnchorToReference ) ) ) )
              if( anc.GetType().IsSubclassOf( typeof( AnchorToReference ) ) )
              {

                AnchorToReference refAnc = anc as AnchorToReference;
                AcObject refEnt = tr.GetObject( refAnc.SingleReferenceId, OpenMode.ForRead );
                msg += "\nto " + TypeAndHandle( refEnt );
              }
            }
            App.MsgBox( msg );
            tr.Commit();
          }
        }
        catch( Exception ex )
        {
          App.MsgBox( "Exception: " + ex.Message );
        }
      }
    }
  }
}
