#region Header
//
// .NET Sample
//
// Copyright (C) 2005-2007 by Autodesk, Inc.
//
// Permission to use, copy, modify, and distribute this software
// for any purpose and without fee is hereby granted, provided
// that the above copyright notice appears in all copies and
// that both that copyright notice and the limited warranty and
// restricted rights notice below appear in all supporting
// documentation.
//
// AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS.
// AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
// MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC.
// DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
// UNINTERRUPTED OR ERROR FREE.
//
// Use, duplication, or disclosure by the U.S. Government is subject to
// restrictions set forth in FAR 52.227-19 (Commercial Computer
// Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
// (Rights in Technical Data and Computer Software), as applicable.
//
#endregion // Header

#region Namespaces
using System.Reflection;

// AutoCAD
using Autodesk.AutoCAD.Runtime;
using Autodesk.AutoCAD.ApplicationServices;
using Autodesk.AutoCAD.DatabaseServices;
using Autodesk.AutoCAD.EditorInput;
using Autodesk.AutoCAD.Geometry;

// ACA (Aec)
using Autodesk.Aec.DatabaseServices;
using Autodesk.Aec.ApplicationServices;
using Autodesk.Aec.Arch.ApplicationServices;
using Autodesk.Aec.Arch.DatabaseServices;
using Autodesk.Aec.Arch.Geometry;

// To resolve Object/Entity ambiguities
using AcObject = Autodesk.AutoCAD.DatabaseServices.DBObject;
using AecObject = Autodesk.Aec.DatabaseServices.DBObject;
using AcEntity = Autodesk.AutoCAD.DatabaseServices.Entity;
using AecEntity = Autodesk.Aec.DatabaseServices.Entity;

// To resolve Application ambiguities
using AcadApp = Autodesk.AutoCAD.ApplicationServices.Application;

// To resolve TransactionManager ambiguities
using DbTrMgr = Autodesk.AutoCAD.DatabaseServices.TransactionManager; //wraps AcDbTransactionManager
using AppTrMgr = Autodesk.AutoCAD.ApplicationServices.TransactionManager; //wraps AcTransactionManager

// To resolve ObjectId ambiguities
using ObjectId = Autodesk.AutoCAD.DatabaseServices.ObjectId;
using ObjectIdCollection = Autodesk.AutoCAD.DatabaseServices.ObjectIdCollection;

#endregion // Namespaces

namespace AcaNetIntroCs
{
  /// <summary>
  /// A class containing commands.
  /// </summary>
  public class Commands2b
  {
    // fixed dimensions in mm
    const double kHouseWidth = 6000;
    const double kHouseDepth = 8000;
    const double kWallHeight = 2500;
    const double kRoofOverhang = 150;
    const double kSlabThickness = 254;

    /// <summary>
    /// Command to create a "house": 4 walls, a door and a window.
    /// </summary>
    [CommandMethod( "TestHouse" )]
    static public void TestHouse()
    {
      try
      {
        Database db = HostApplicationServices.WorkingDatabase;
        Editor ed = AcadApp.DocumentManager.MdiActiveDocument.Editor;
        using( Transaction tr = db.TransactionManager.StartTransaction() )
        {
          //
          // simple error checking where no exceptions are caught in the helpers
          // and only the success flag is passed back from functions.
          // Note that transaction will always be disposed.
          //
          double scaleFactor = 1.0; // scale factor for any current units
          Point3d centre = new Point3d(); // centre point for the house
          Wall[] walls = new Wall[4];
          if( GetScaleFactor( ref scaleFactor, db, tr )
            && GetCentrePoint( ref centre, ed )
            && CreateWalls( ref walls, centre, scaleFactor, db, tr )
            && CreateDoor( walls[0], db, tr ) // create and anchor a door (all defaults)
            && CreateWindow( walls[2], db, tr, scaleFactor ) // create and anchor a window (specific properties including the style and anchor)
            && CreateRoof( centre, scaleFactor, db, tr ) ) // create a roof over the walls
          {
            tr.Commit();
          }
        }
      }
      catch( Exception ex )
      {
        App.MsgBox( "Exception: " + ex.Message );
      }
    }

    /// <summary>
    /// Generic AutoCAD.NET helper to add an entity to database model space and transaction.
    /// </summary>
    static public void AddToDbAndTransaction( AcEntity ent, Database db, string blkName, Transaction tr )
    {
      BlockTable bt = tr.GetObject( db.BlockTableId, OpenMode.ForRead ) as BlockTable;
      BlockTableRecord btr = tr.GetObject( bt[BlockTableRecord.ModelSpace], OpenMode.ForWrite ) as BlockTableRecord;
      btr.AppendEntity( ent );
      tr.AddNewlyCreatedDBObject( ent, true );
    }
    
    
    /// <summary>
    /// Generic ACA.NET helper to get the scale factor wrt millimeters according to the current units
    /// </summary>
    /// <param name="scaleFactor"></param>
    /// <param name="db"></param>
    /// <param name="tr"></param>
    /// <returns>True on success, and sets scaleFactor.</returns>
    static public bool GetScaleFactor( ref double scaleFactor, Database db, Transaction tr )
    {
      ObjectId idDwgVars = DrawingSetupVariables.GetInstance( db, false );
      DrawingSetupVariables dwgVars = tr.GetObject( idDwgVars, OpenMode.ForWrite) as DrawingSetupVariables;
      switch( dwgVars.LinearUnit )
      {
        case Autodesk.Aec.BuiltInUnit.Inch:
          scaleFactor = 1 / 25.4;
          break;
        case Autodesk.Aec.BuiltInUnit.Foot:
          scaleFactor = 1 / 304.8;
          break;
        case Autodesk.Aec.BuiltInUnit.Millimeter:
          scaleFactor = 1;
          break;
        case Autodesk.Aec.BuiltInUnit.Centimeter:
          scaleFactor = 0.1;
          break;
        case Autodesk.Aec.BuiltInUnit.Decimeter:
          scaleFactor = 0.01;
          break;
        case Autodesk.Aec.BuiltInUnit.Meter:
          scaleFactor = 0.001;
          break;
        default:
          return false;
      }
      return true;
    }
    
    /// <summary>
    /// Specific helper to select centre point.
    /// </summary>
    static private bool GetCentrePoint( ref Point3d pt, Editor ed )
    {
      PromptPointResult resPoint = ed.GetPoint( "Select centre point: " );
      bool rc = ( PromptStatus.OK == resPoint.Status );
      if( rc )
      {
        pt = resPoint.Value;
      }
      return rc;
    }
    
    /// <summary>
    /// Specific helper to create 4 walls for house.
    /// </summary>
    /// <param name="walls"></param>
    /// <param name="centre"></param>
    /// <param name="scaleFactor"></param>
    /// <param name="db"></param>
    /// <param name="tr"></param>
    /// <returns></returns>
    static private bool CreateWalls( ref Wall[] walls, Point3d centre, double scaleFactor, Database db, Transaction tr )
    {
      double halfWidth = 0.5 * scaleFactor * kHouseWidth;
      double halfDepth = 0.5 * scaleFactor * kHouseDepth;
      Point3d[] corners = new Point3d[4];
      corners[0] = new Point3d( centre.X - halfWidth, centre.Y - halfDepth, centre.Z );
      corners[1] = new Point3d( centre.X + halfWidth, centre.Y - halfDepth, centre.Z );
      corners[2] = new Point3d( centre.X + halfWidth, centre.Y + halfDepth, centre.Z );
      corners[3] = new Point3d( centre.X - halfWidth, centre.Y + halfDepth, centre.Z );
      // loop to create 4 walls
      for( int iWall = 0; iWall < 4; ++iWall )
      {
        int iEnd = (3 > iWall) ? (iWall + 1) : 0;
        walls[iWall] = new Wall();
        walls[iWall].SetDatabaseDefaults( db );
        walls[iWall].SetToStandard( db );
        walls[iWall].SetDefaultLayer();
        walls[iWall].Set( corners[iWall], corners[iEnd], Vector3d.ZAxis );
        walls[iWall].BaseHeight = scaleFactor * kWallHeight;
        AddToDbAndTransaction( walls[iWall], db, BlockTableRecord.ModelSpace, tr );
      }
      return true;
    }
    
    /// <summary>
    /// Specific helper to create a door and anchor it to given wall.
    /// </summary>
    static private bool CreateDoor( Wall wall, Database db, Transaction tr )
    {
        Door door = new Door();
        door.SetDatabaseDefaults( db );
        door.SetToStandard( db );
        door.SetDefaultLayer();

        AddToDbAndTransaction( door, db, BlockTableRecord.ModelSpace, tr );
        AnchorOpeningBaseToWall anc = new AnchorOpeningBaseToWall();
        anc.SubSetDatabaseDefaults( db );
        anc.SetToStandard( db );
        door.SetAnchor( anc );
        anc.CurveId = wall.Id;
        return true;
    }
    
    /// <summary>
    /// Specific helper to get or create specific window style
    /// </summary>
    static private ObjectId GetOrCreateWindowStyle( string name, double scalefactor, Database db, Transaction tr )
    {
      // fixed dimensions in mm
      const double kGlassThick = 15;
      WindowStyle style;
      DictionaryWindowStyle dict = new DictionaryWindowStyle( db );
      ObjectId id;
      // existing style
      if( dict.Has( name, tr ) )
      {
        id = dict.GetAt( name );
      }
      else
      {
        // create new style
        style = new WindowStyle();
        style.SubSetDatabaseDefaults( db );
        style.SetToStandard( db );
        // some specific properties:
        style.UseDepthOfWall = true;
        style.WindowShape = OpenShapeType.Oval;
        style.WindowType = WindowType.DoubleHung;
        style.GlassThickness = kGlassThick * scalefactor;
        // add to dict and transaction
        dict.AddNewRecord( name, style ); // DictionaryRecord
        tr.AddNewlyCreatedDBObject( style, true );
        id = style.Id;
      }
      return id;
    }
    
    /// <summary>
    /// Specific helper to create a window and anchor it to given wall.
    /// </summary>
    static private bool CreateWindow( Wall wall, Database db, Transaction tr, double scaleFactor )
    {
      bool rc = false;
      // fixed dimensions in mm
      const double kWindowWidth = 3000;
      const double kWindowHeight = 1500;
      // create window
      Window window = new Window();
      window.SetDatabaseDefaults( db );
      window.SetToStandard( db );
      window.SetDefaultLayer();

      // specific properties:
      window.Width = kWindowWidth * scaleFactor;
      window.Height = kWindowHeight * scaleFactor;
      AddToDbAndTransaction( window, db, BlockTableRecord.ModelSpace, tr );
      ObjectId styleId = GetOrCreateWindowStyle( "AcaNetIntro", scaleFactor, db, tr );
      if( !styleId.IsNull )
      {
        window.StyleId = styleId;
        // Create anchor
        AnchorOpeningBaseToWall anc = new AnchorOpeningBaseToWall();
        anc.SubSetDatabaseDefaults( db );
        anc.SetToStandard( db );
        // specific properties (center the window within the wall):
        anc.AnchorX.OffsetType = CurveXOffsetType.FromMidpointOfCurve;
        anc.AnchorX.OffsetDistance = 0;
        anc.AnchorX.MeasureToType = CurveXMeasureToType.ToCenter;
        anc.AnchorZ.OffsetFromType = CurveZOffsetFromType.FromCenter;
        anc.AnchorZ.OffsetDistance = 0;
        anc.AnchorZ.OffsetToType = CurveZOffsetToType.ToCenter;
        // set owner and reference
        window.SetAnchor( anc );
        anc.CurveId = wall.Id;
        rc = true;
      }
      return rc;
    }
    
    /// <summary>
    /// Create a roof over four walls.
    /// </summary>
    static private bool CreateRoof( Point3d centre, double scaleFactor, Database db, Transaction tr )
    {
      double halfWidth = 0.5 * scaleFactor * kHouseWidth;
      double halfDepth = 0.5 * scaleFactor * kHouseDepth;
      double wallHeight = scaleFactor * kWallHeight;

      double overhang = scaleFactor * kRoofOverhang;
      double slabThickness = scaleFactor * kSlabThickness;

      // calculate four corners of a roof
      Point2d[] corners = new Point2d[4];
      corners[0] = new Point2d( -halfWidth, -halfDepth );
      corners[1] = new Point2d( +halfWidth, -halfDepth );
      corners[2] = new Point2d( +halfWidth, +halfDepth );
      corners[3] = new Point2d( -halfWidth, +halfDepth );
      // define a polyline from them
      Polyline ipline = new Polyline( 4 );
      ipline.AddVertexAt( 0, corners[0], 0, 1, 1 );
      ipline.AddVertexAt( 1, corners[1], 0, 1, 1 );
      ipline.AddVertexAt( 2, corners[2], 0, 1, 1 );
      ipline.AddVertexAt( 3, corners[3], 0, 1, 1 );
      ipline.Closed = true;
      Matrix3d mat = Matrix3d.Displacement( centre.GetAsVector() );
      ipline.TransformBy( mat );
      // create a roof object 
      Roof roof = new Roof();
      roof.SetDatabaseDefaults( db );
      roof.SetToStandard( db );
      roof.SetDefaultLayer();

      // set the roof profile 
      RoofProfile roofProf = roof.RoofProfile;
      roofProf.ConvertPolylineToRoofContour( ipline, Tolerance.Global );
      bool b = roofProf.IsValid( true, Tolerance.Global );
      roof.RoofProfile = roofProf;

      // set the roof overhang
      roof.Overhang = overhang;

      roof.SetThickness(slabThickness, ThicknessDirection.Roof, Tolerance.Global.EqualPoint);

      Point3d p = new Point3d( 0, 0, centre.Z + wallHeight );
      roof.SetLocation( p, p, Vector3d.ZAxis, Tolerance.Global );
      roof.CalculateRoof();
      // add it to the model space
      AddToDbAndTransaction( roof, db, BlockTableRecord.ModelSpace, tr );
      return true;
    }
  }
}
