#Region "Header"
'      .NET Sample
'
'      Copyright (C) 2005-2007 by Autodesk, Inc.
'
'      Permission to use, copy, modify, and distribute this software
'      for any purpose and without fee is hereby granted, provided
'      that the above copyright notice appears in all copies and
'      that both that copyright notice and the limited warranty and
'      restricted rights notice below appear in all supporting
'      documentation.
'
'      AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS.
'      AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
'      MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC.
'      DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
'      UNINTERRUPTED OR ERROR FREE.
'
'      Use, duplication, or disclosure by the U.S. Government is subject to
'      restrictions set forth in FAR 52.227-19 (Commercial Computer
'      Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
'      (Rights in Technical Data and Computer Software), as applicable.
#End Region

#Region "Imports"

Imports System.Reflection
'Imports System.Windows.Forms
Imports System.Collections.Specialized
' AutoCAD
Imports Autodesk
Imports Autodesk.AutoCAD.Runtime
Imports Autodesk.AutoCAD.ApplicationServices
Imports Autodesk.AutoCAD.DatabaseServices
Imports Autodesk.AutoCAD.EditorInput
Imports Autodesk.AutoCAD.Geometry
' ACA (Aec)
Imports Autodesk.Aec.DatabaseServices
Imports Autodesk.Aec.Arch.DatabaseServices
Imports Autodesk.Aec.ApplicationServices
Imports Autodesk.Aec.Arch.ApplicationServices
Imports Autodesk.AEC.Arch.Geometry

' To resolve Object/Entity ambiguities
Imports AcObject = Autodesk.AutoCAD.DatabaseServices.DBObject
Imports AecObject = Autodesk.Aec.DatabaseServices.DBObject
Imports AcEntity = Autodesk.AutoCAD.DatabaseServices.Entity
Imports AecEntity = Autodesk.Aec.DatabaseServices.Entity

' To resolve Application ambiguities
'Imports AcadApp = Autodesk.AutoCAD.ApplicationServices.Application

' To resolve ObjectId ambiguities
Imports ObjectId = Autodesk.AutoCAD.DatabaseServices.ObjectId
Imports ObjectIdCollection = Autodesk.AutoCAD.DatabaseServices.ObjectIdCollection

#End Region

'A class containing commands
Public Class Commands2b

    ' fixed dimensions in mm
    Const kHouseWidth As Double = 6000
    Const kHouseDepth As Double = 8000
    Const kWallHeight As Double = 2500
    Const kRoofOverhang As Double = 150
    Const kSlabThickness As Double = 254

    ' Command to create a "house": 4 walls, a door and a window. 
    <CommandMethod("AcaNetIntroVb", "AcaNetCreateHouse", CommandFlags.Modal)> _
    Public Sub HouseAdd()

        ' Declare once here and pass to all helpers
        Dim db As Database = HostApplicationServices.WorkingDatabase
        Dim ed As Editor = Application.DocumentManager.MdiActiveDocument.Editor
        Dim tr As Transaction = db.TransactionManager.StartTransaction

        'Let's use simple error-checking where any specific exceptions are caught in the helpers
        '  while only the success flag is passed back from Functions.
        '  Note that Finally block is executed even if Exit Sub called from Try block
        Try

            ' Scale factor for any current units
            Dim scaleFactor As Double = 1
            If Not GetScaleFactor(scaleFactor, db, tr) Then Exit Sub

            ' Get centre point for the house
            Dim centre As Point3d
            If Not GetCentrePoint(centre, ed) Then Exit Sub

            ' Create Walls
            Dim walls(3) As Wall
            If Not CreateWalls(walls, centre, scaleFactor, db, tr) Then Exit Sub

            ' Create and anchor a Door (all defaults)
            If Not CreateDoor(walls(0), db, tr) Then Exit Sub

            ' Create and anchor a Window (specific properties including the style and anchor)
            If Not CreateWindow(walls(2), db, tr, scaleFactor) Then Exit Sub

            ' Create a roof over the walls
            If Not CreateRoof(centre, scaleFactor, db, tr) Then Exit Sub

            tr.Commit()
        Catch
            tr.Abort()
        Finally
            tr.Dispose()
        End Try

    End Sub

    ' Generic AutoCAD.NET helper to add an entity to Db and Transaction
    Public Function AddToDbAndTransaction(ByRef ent As AcEntity, ByRef db As Database, ByVal blkName As String, ByRef tr As Transaction) As Boolean

        Try
            Dim bt As BlockTable = CType(tr.GetObject(db.BlockTableId, OpenMode.ForRead), BlockTable)
            Dim btr As BlockTableRecord = _
             CType(tr.GetObject(bt(BlockTableRecord.ModelSpace), OpenMode.ForWrite), BlockTableRecord)
            btr.AppendEntity(ent)
            tr.AddNewlyCreatedDBObject(ent, True)
        Catch ex As Exception
            MsgBox("Exception in AddToDbAndTransaction: " + ex.Message, "ACA.NET Intro")
            Return False
        End Try

        'OK if here
        Return True

    End Function

    ' Generic ACA.NET helper to get the scale factor wrt millimeters according to the current units
    Public Function GetScaleFactor(ByRef scaleFactor As Double, ByRef db As Database, ByRef tr As Transaction) As Boolean

        Try
            Dim idDwgVars As ObjectId = DrawingSetupVariables.GetInstance(db, False)
            Dim dwgVars As DrawingSetupVariables = CType(tr.GetObject(idDwgVars, OpenMode.ForWrite), DrawingSetupVariables)

            Select Case dwgVars.LinearUnit

                Case Autodesk.Aec.BuiltInUnit.Inch
                    scaleFactor = 1 / 25.4
                Case Autodesk.Aec.BuiltInUnit.Foot
                    scaleFactor = 1 / 304.8
                Case Autodesk.Aec.BuiltInUnit.Millimeter
                    scaleFactor = 1.0#
                Case Autodesk.Aec.BuiltInUnit.Centimeter
                    scaleFactor = 0.1
                Case Autodesk.Aec.BuiltInUnit.Decimeter
                    scaleFactor = 0.01
                Case Autodesk.Aec.BuiltInUnit.Meter
                    scaleFactor = 0.001
                Case Else
                    Return False

            End Select

        Catch ex As Exception
            MsgBox("Exception in GetScaleFactor: " + ex.Message, "ACA.NET Intro")
            Return False
        End Try

        'OK if here
        Return True

    End Function

    ' Specific helper to select centre-point
    Private Function GetCentrePoint(ByRef pt As Point3d, ByRef ed As Editor) As Boolean

        Dim resPoint As PromptPointResult = ed.GetPoint("Select centre point")
        If resPoint.Status <> PromptStatus.OK Then Return False
        pt = resPoint.Value

        'OK if here
        Return True
    End Function

    ' Specific helper to create 4 walls for this house
    Private Function CreateWalls(ByRef walls() As Wall, ByVal centre As Point3d, ByVal scaleFactor As Double, ByRef db As Database, ByRef tr As Transaction) As Boolean

        Try
            ' Calculate 4 corners
            Dim corners(3) As Point3d
            corners(0) = New Point3d(centre.X - scaleFactor * kHouseWidth / 2, centre.Y - scaleFactor * kHouseDepth / 2, centre.Z)
            corners(1) = New Point3d(centre.X + scaleFactor * kHouseWidth / 2, centre.Y - scaleFactor * kHouseDepth / 2, centre.Z)
            corners(2) = New Point3d(centre.X + scaleFactor * kHouseWidth / 2, centre.Y + scaleFactor * kHouseDepth / 2, centre.Z)
            corners(3) = New Point3d(centre.X - scaleFactor * kHouseWidth / 2, centre.Y + scaleFactor * kHouseDepth / 2, centre.Z)

            ' Loop-create 4 walls
            Dim iWall As Integer
            For iWall = 0 To 3
                Dim iEnd As Integer = iWall + 1
                If iEnd > 3 Then iEnd = 0
                walls(iWall) = New Wall()
                walls(iWall).SetDatabaseDefaults(db)
                walls(iWall).SetToStandard(db)
                walls(iWall).SetDefaultLayer()
                walls(iWall).Set(corners(iWall), corners(iEnd), Vector3d.ZAxis)
                walls(iWall).BaseHeight = scaleFactor * kWallHeight
                If Not AddToDbAndTransaction(CType(walls(iWall), AcEntity), db, BlockTableRecord.ModelSpace, tr) Then Return False
            Next

        Catch ex As Exception
            MsgBox("Exception in CreateWalls: " + ex.Message, "ACA.NET Intro")
            Return False
        End Try

        'OK if here
        Return True
    End Function

    ' Specific helper to create a door and anchor it to given wall
    Private Function CreateDoor(ByRef wall As Wall, ByRef db As Database, ByRef tr As Transaction) As Boolean

        Try
            ' Create default door
            Dim door As New Door()
            door.SetDatabaseDefaults(db)
            door.SetToStandard(db)
            door.SetDefaultLayer()

            If Not AddToDbAndTransaction(CType(door, AcEntity), db, BlockTableRecord.ModelSpace, tr) Then Return False

            ' Create default anchor
            Dim anc As New AnchorOpeningBaseToWall()
            anc.SubSetDatabaseDefaults(db)
            anc.SetToStandard(db)
            door.SetAnchor(anc)
            anc.CurveId = wall.Id

        Catch ex As Exception
            MsgBox("Exception in CreateDoor: " + ex.Message, "ACA.NET Intro")
            Return False
        End Try

        'OK if here
        Return True
    End Function

    ' Specific helper to get-or-create specific window style
    Private Function GetOrCreateWindowStyle(ByVal name As String, ByVal scalefactor As Double, ByRef db As Database, ByRef tr As Transaction) As ObjectId

        ' Fixed Dimensions in mm
        Const kGlassThick As Double = 15

        Dim style As WindowStyle
        Try
            Dim dict As New DictionaryWindowStyle(db)

            ' Existing style
            If dict.Has(name, tr) Then
                Return dict.GetAt(name)
            End If

            'Create new style
            style = New WindowStyle()
            style.SubSetDatabaseDefaults(db)
            style.SetToStandard(db)
            ' some specific properties:
            style.UseDepthOfWall = True
            style.WindowShape = OpenShapeType.Oval
            style.WindowType = WindowType.DoubleHung
            style.GlassThickness = kGlassThick * scalefactor
            ' add to dict and transaction
            dict.AddNewRecord(name, CType(style, DictionaryRecord))
            tr.AddNewlyCreatedDBObject(CType(style, AcObject), True)

            Return style.Id

        Catch ex As Exception
            MsgBox("Exception in GetOrCreateWindowStyle: " + ex.Message, "ACA.NET Intro")
            Return ObjectId.Null
        End Try

    End Function

    ' Specific helper to create a window and anchor it to given wall
    Private Function CreateWindow(ByRef wall As Wall, ByRef db As Database, ByRef tr As Transaction, ByVal scaleFactor As Double) As Boolean

        ' Fixed Dimensions in mm
        Const kWindowWidth As Double = 3000
        Const kWindowHeight As Double = 1500

        Try
            ' Create window
            Dim window As New Window()
            window.SetDatabaseDefaults(db)
            window.SetToStandard(db)
            window.SetDefaultLayer()

            ' specific properties:
            window.Width = kWindowWidth * scaleFactor
            window.Height = kWindowHeight * scaleFactor
            If Not AddToDbAndTransaction(CType(window, AcEntity), db, BlockTableRecord.ModelSpace, tr) Then Return False

            ' Get-or-create specific window style 
            Dim styleId As ObjectId = GetOrCreateWindowStyle("AcaNetIntro", scaleFactor, db, tr)
            If (styleId.IsNull) Then
                Return False
            Else
                window.StyleId = styleId
            End If

            ' Create anchor
            Dim anc As New AnchorOpeningBaseToWall()
            anc.SubSetDatabaseDefaults(db)
            anc.SetToStandard(db)
            ' specific properties (center the window within the wall):
            anc.AnchorX.OffsetType = CurveXOffsetType.FromMidpointOfCurve
            anc.AnchorX.OffsetDistance = 0
            anc.AnchorX.MeasureToType = CurveXMeasureToType.ToCenter
            anc.AnchorZ.OffsetFromType = CurveZOffsetFromType.FromCenter
            anc.AnchorZ.OffsetDistance = 0
            anc.AnchorZ.OffsetToType = CurveZOffsetToType.ToCenter
            ' set owner and reference
            window.SetAnchor(anc)
            anc.CurveId = wall.Id

        Catch ex As Exception
            MsgBox("Exception in CreateWindow: " + ex.Message, "ACA.NET Intro")
            Return False
        End Try

        'OK if here
        Return True
    End Function

    ' create a roof over four walls. 
    Private Function CreateRoof(ByVal centre As Point3d, ByVal scaleFactor As Double, ByRef db As Database, ByRef tr As Transaction) As Boolean

        Dim overhang As Double = scaleFactor * kRoofOverhang
        Dim slabThickness As Double = scaleFactor * kSlabThickness
        Dim wallHeight As Double = 2500 * scaleFactor

        Try

            ' calculate four corners of a roof
            Dim corners(3) As Point2d
            corners(0) = New Point2d(0 - scaleFactor * kHouseWidth / 2, 0 - scaleFactor * kHouseDepth / 2)
            corners(1) = New Point2d(0 + scaleFactor * kHouseWidth / 2, 0 - scaleFactor * kHouseDepth / 2)
            corners(2) = New Point2d(0 + scaleFactor * kHouseWidth / 2, 0 + scaleFactor * kHouseDepth / 2)
            corners(3) = New Point2d(0 - scaleFactor * kHouseWidth / 2, 0 + scaleFactor * kHouseDepth / 2)

            ' define a polyline from them
            Dim ipline As Polyline = New Polyline(4)
            ipline.AddVertexAt(0, corners(0), 0, 1, 1)
            ipline.AddVertexAt(1, corners(1), 0, 1, 1)
            ipline.AddVertexAt(2, corners(2), 0, 1, 1)
            ipline.AddVertexAt(3, corners(3), 0, 1, 1)
            ipline.Closed = True

            ' align the roof location with the wall
            Dim mat As Matrix3d = New Matrix3d()
            mat = Matrix3d.Displacement(centre.GetAsVector)

            ipline.TransformBy(mat)

            ' create a roof object 
            Dim roof As Roof = New Roof()
            roof.SetDatabaseDefaults(db)
            roof.SetToStandard(db)
            roof.SetDefaultLayer()

            ' set the roof profile 
            Dim roofProf As RoofProfile = roof.RoofProfile
            roofProf.ConvertPolylineToRoofContour(ipline, Tolerance.Global)

            Dim b As Boolean = roofProf.IsValid(True, Tolerance.Global)

            roof.RoofProfile = roofProf

            roof.Overhang = overhang
            roof.SetThickness(slabThickness, ThicknessDirection.Roof, Tolerance.Global.EqualPoint)

            roof.SetLocation(New Point3d(0.0, 0.0, centre.Z + wallHeight), New Point3d(0.0, 0.0, centre.Z + wallHeight), Vector3d.ZAxis, Tolerance.Global)
            roof.CalculateRoof()

            ' add it to the model space
            If Not AddToDbAndTransaction(CType(roof, AcEntity), db, BlockTableRecord.ModelSpace, tr) Then Return False

        Catch ex As Exception
            MsgBox("Exception in CreateRoof: " + ex.Message, "ACA.NET Intro")
            Return False
        End Try

        'OK if here
        Return True

    End Function


End Class

