// (C) Copyright 2006 by Autodesk, Inc. 
//
// Permission to use, copy, modify, and distribute this software in
// object code form for any purpose and without fee is hereby granted, 
// provided that the above copyright notice appears in all copies and 
// that both that copyright notice and the limited warranty and
// restricted rights notice below appear in all supporting 
// documentation.
//
// AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS. 
// AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
// MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC. 
// DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
// UNINTERRUPTED OR ERROR FREE.
//
// Use, duplication, or disclosure by the U.S. Government is subject to 
// restrictions set forth in FAR 52.227-19 (Commercial Computer
// Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
// (Rights in Technical Data and Computer Software), as applicable.
//


using System;
using System.Collections.Generic;
using System.Text;
using System.Diagnostics;
using System.Threading;

using Autodesk.AutoCAD.DatabaseServices;
using Autodesk.AutoCAD.Geometry;
using Autodesk.AutoCAD.EditorInput;
using AcDb = Autodesk.AutoCAD.DatabaseServices;

using AecDb = Autodesk.Aec.DatabaseServices;
using AecGe = Autodesk.Aec.Geometry;
using Autodesk.Aec.Arch.Geometry;
using Autodesk.Aec.DatabaseServices;
using Autodesk.Aec.Arch.DatabaseServices;

using AecAppSrvcs = Autodesk.Aec.ApplicationServices;
using ArchDb = Autodesk.Aec.Arch.DatabaseServices;
using AcadApp = Autodesk.AutoCAD.ApplicationServices.Application;

using MgdDbg;
using MgdDbg.Test;
using MgdDbg.Utils;

namespace MgdDbgAec.ObjTests
{

    class HardwireBuilding : MgdDbgTestFuncs
    {
        private Database m_db = null;
        private CompBldrCurSpace m_compBldr = null;

        public
        HardwireBuilding ()
        {
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Office Building", "Hardwire a simple building", "Hardwired Buildings", new MgdDbgTestFuncInfo.TestFunc(Building1), MgdDbgTestFuncInfo.TestType.Create));
        }

        public void
        Building1 ()
        {
            m_db = MgdDbg.Utils.Db.GetCurDwg();

            AcDb.ObjectIdCollection extWallIds = new AcDb.ObjectIdCollection();
            Point2dCollection outlinePts = new Point2dCollection();

            using (m_compBldr = new CompBldrCurSpace(m_db)) {
                m_compBldr.Start();
                m_compBldr.PushXform(MgdDbg.Utils.Db.GetUcsMatrix(m_db));
                
                outlinePts = Building1_OutlinePoints();
                int count = outlinePts.Count;
                
                ///
                /// Make the front skin of the building with curtain walls
                /// 
                for (int i = 0; i < outlinePts.Count - 4; i++, count--) {

                    CreateCurtainWall(AcDb.ObjectId.Null, outlinePts[i], outlinePts[i + 1]);
                }

                ///
                /// Create rest of the exterior walls and windows
                /// 
                for (int i = count + 1; i < outlinePts.Count - 1; i++) {

                    AcDb.ObjectId wallId = CreateWall(AcDb.ObjectId.Null, outlinePts[i], outlinePts[i + 1]);
                    CreateExteriorWindows(wallId);
                }

                ///
                /// Create floors with holes for stairwells
                /// 
                for (int i = 0; i < 3; i++) {

                    AcDb.ObjectId floorSlabId = CreateFloorSlab(AcDb.ObjectId.Null,
                                                ConvertTo3d(Building1_OutlinePoints(), 120.0 * i));

                    /// 1st floor should'nt get a hole
                    if (i == 0)
                        continue;

                    /// floor in the slabs for stairwell
                    CreateFloorSlabHole(floorSlabId, ConvertTo3d(SlabHolePoints(0.0), 120.0 * i));
                    CreateFloorSlabHole(floorSlabId, ConvertTo3d(SlabHolePoints(650.0), 120.0 * i));
                }

                ///
                /// Create roof
                /// 
                CreateRoofSlab(AcDb.ObjectId.Null, ConvertTo3d(Building1_OutlinePoints(), 360.0));

                ///
                /// Create stairs and railings
                ///
                for (int i = 0; i < 4; i++) {

                    AcDb.ObjectId stairId = new AcDb.ObjectId();

                    /// first two stairs on the left are clockwise
                    if (i < 2) {
                        stairId = CreateStair(SlabHolePoints(0)[3], SlabHolePoints(0)[2],
                                              120.0 * i, 120.0, false);

                        /// the railing anchored to the stair
                        CreateRailing(stairId, false);

                        /// the safety railing at the floor level
                        CreateRailing(new Point2d(SlabHolePoints(0)[1].X, SlabHolePoints(0)[1].Y),
                                      new Point2d(SlabHolePoints(0)[2].X, SlabHolePoints(0)[2].Y), 120 * (i + 1));

                        /// the safety railing for the last flight of each stair
                        CreateRailing(stairId, 2, false);
                    }
                    /// next two on the right are counter clockwise
                    else {
                        stairId = CreateStair(SlabHolePoints(650)[2], SlabHolePoints(650)[3],
                                               120.0 * (i - 2), 120.0, true);

                        /// the railing anchored to the stair
                        CreateRailing(stairId, true);

                        /// the safety railing at the floor level
                        CreateRailing(new Point2d(SlabHolePoints(650)[3].X, SlabHolePoints(650)[3].Y),
                                      new Point2d(SlabHolePoints(650)[4].X, SlabHolePoints(650)[4].Y), 120 * ((i - 2) + 1));

                        /// the safety railing for the last flight of each stair
                        CreateRailing(stairId, 2, true);
                    }
                }

                ///
                /// the safety railing at the floor level
                /// 
                for (int i = 0; i < 2; i++) {
                    
                    CreateRailing(new Point2d(SlabHolePoints(i * 650)[3].X + 40, SlabHolePoints(i * 650)[3].Y),
                                            new Point2d(SlabHolePoints(i * 650)[2].X - 40, SlabHolePoints(i * 650)[2].Y), 120);
                    
                }

                ///
                /// safety railing at top floors
                /// 
                CreateRailing(new Point2d(SlabHolePoints(0)[3].X, SlabHolePoints(0)[3].Y),
                              new Point2d(SlabHolePoints(0)[2].X - 40, SlabHolePoints(0)[2].Y), 240);

                CreateRailing(new Point2d(SlabHolePoints(650)[3].X + 40, SlabHolePoints(650)[3].Y),
                              new Point2d(SlabHolePoints(650)[2].X, SlabHolePoints(650)[2].Y), 240);

                m_compBldr.Commit();
            }

            AcadApp.DocumentManager.MdiActiveDocument.SendStringToExecute("Zoom e\n", false, false, false);

        }

        private AcDb.ObjectId
        CreateCurtainWall (AcDb.ObjectId curtainWallStyleId, Point2d startPt, Point2d endPt)
        {
            ArchDb.CurtainWallLayout curtainWall = new ArchDb.CurtainWallLayout();
            curtainWall.BaseHeight = 360.0;
            curtainWall.SetLinearBaseCurve(new Point3d(startPt.X, startPt.Y, 0.0),
                                           new Point3d(endPt.X, endPt.Y, 0.0),
                                           Vector3d.ZAxis);

            if (curtainWallStyleId != AcDb.ObjectId.Null)   // if its null, SetDatabaseDefaults will set it to default
                curtainWall.StyleId = curtainWallStyleId;

            m_compBldr.AddToDb(curtainWall);

            return curtainWall.ObjectId;
        }

        private AcDb.ObjectId
        CreateWall (AcDb.ObjectId wallStyleId, Point2d startPt, Point2d endPt)
        {
            ArchDb.Wall wall = new ArchDb.Wall();
            wall.BaseHeight = 360.0;
            wall.Set(new Point3d(startPt.X, startPt.Y, 0.0), new Point3d(endPt.X, endPt.Y, 0.0), Vector3d.ZAxis);

            if (wallStyleId != AcDb.ObjectId.Null)   // if its null, SetDatabaseDefaults will set it to default
                wall.StyleId = wallStyleId;

            m_compBldr.AddToDb(wall);

            return wall.ObjectId;
        }

        private AcDb.ObjectId
        CreateFloorSlab (AcDb.ObjectId floorSlabStyleId, Point3dCollection pts)
        {
            ArchDb.Slab slab = new ArchDb.Slab();
            slab.BaseThickness = 4.0;
            slab.SetBaseCurve(pts, Vector3d.ZAxis, true);

            if (floorSlabStyleId != AcDb.ObjectId.Null)   // if its null, SetDatabaseDefaults will set it to default
                slab.StyleId = floorSlabStyleId;

            m_compBldr.AddToDb(slab);

            return slab.ObjectId;
        }

        private AcDb.ObjectId
        CreateFloorSlabHole (AcDb.ObjectId floorSlabId, Point3dCollection pts)
        {
            // need to commit changes before opening a slab object to write
            m_compBldr.Commit();
            m_compBldr.Start();
            ArchDb.Slab slab = m_compBldr.Transaction.GetObject(floorSlabId, OpenMode.ForWrite) as ArchDb.Slab;
            slab.AddHole(pts, Vector3d.ZAxis, Tolerance.Global);
            
            return slab.ObjectId;
        }


        private AcDb.ObjectId
        CreateRoofSlab (AcDb.ObjectId roofSlabStyleId, Point3dCollection pts)
        {
            ArchDb.RoofSlab roofSlab = new ArchDb.RoofSlab();
            roofSlab.BaseThickness = 4.0;
            roofSlab.SetBaseCurve(pts, Vector3d.ZAxis, true);

            if (roofSlabStyleId != AcDb.ObjectId.Null)   // if its null, SetDatabaseDefaults will set it to default
                roofSlab.StyleId = roofSlabStyleId;

            m_compBldr.AddToDb(roofSlab);

            return roofSlab.ObjectId;
        }

        private AcDb.ObjectId
        CreateStair (Point2d startPt, Point2d endPt, 
                     double storeyLevel, double storeyHeight, bool isCounterClockwise)
        {
            ArchDb.Stair stair = new ArchDb.Stair();

            try {
                /// very imp. to initialise to realistic values
                stair.SetToStandard(m_db);

                StairCreatorUShaped creator = new StairCreatorUShaped();
                creator.SetToStandard(m_db);
                creator.SubSetDatabaseDefaults(m_db);

                /// define the path of the stair
                if (isCounterClockwise) {
                    creator.Counterclockwise = true;
                    creator.Start = new Point2d(startPt.X - stair.Width / 2, startPt.Y);
                    creator.End = new Point2d(endPt.X + stair.Width / 2, endPt.Y);
                }
                else {
                    creator.Counterclockwise = false;
                    creator.Start = new Point2d(startPt.X + stair.Width / 2, startPt.Y);
                    creator.End = new Point2d(endPt.X - stair.Width / 2, endPt.Y);
                }

                stair.Creator = creator;

                stair.Location = new Point3d(0, 0, storeyLevel);

                stair.CalculatorData.StoreyHeight = storeyHeight;
                stair.CalculatorData.Flags = StairCalculatorFlags.StoreyHeight ^ StairCalculatorFlags.Tread;

                stair.Create();

                m_compBldr.AddToDb(stair);
            }
            catch (Exception e) {
                System.Windows.Forms.MessageBox.Show(e.Message);
            }

            return stair.ObjectId;
        }

        private AcDb.ObjectId
        CreateRailing (AcDb.ObjectId stairId, bool isCounterClockwise)
        {
            Transaction tr = m_compBldr.Transaction;
            ArchDb.Stair stair = tr.GetObject(stairId, OpenMode.ForRead) as ArchDb.Stair;

            ArchDb.Railing railing = new Railing();
            railing.SetToStandard(m_db);

            // get an anchor to attach to
            AnchorRailingToStair anchor = new AnchorRailingToStair();
            anchor.SetToStandard(m_db);
            anchor.StairId = stairId;

            if (isCounterClockwise)
                anchor.Justification = StairJustificationType.Left;
            else
                anchor.Justification = StairJustificationType.Right;

            railing.SetAnchor(anchor);
            m_compBldr.AddToDb(railing);

            return railing.Id;
        }

        private AcDb.ObjectId
        CreateRailing (Point2d startPt, Point2d endPt, double elevation)
        {
            ArchDb.Railing railing = new Railing();
            railing.SetToStandard(m_db);

            Autodesk.Aec.Geometry.Polyline2d path = new Autodesk.Aec.Geometry.Polyline2d();
            path.AddSegment(new Autodesk.Aec.Geometry.Segment2d(startPt, endPt), true);

            railing.SetPath(path);
            railing.Location = new Point3d(0, 0, elevation);

            m_compBldr.AddToDb(railing);

            return railing.Id;
        }

        private AcDb.ObjectId
        CreateRailing (AcDb.ObjectId stairId, int componentIndex , bool isCounterClockwise)
        {
            ArchDb.Railing railing = new Railing();
            railing.SetToStandard(m_db);
            railing.SetDatabaseDefaults(m_db);

            /// TBD: extend the railing to the end for safety
            DictionaryRailingStyle dict = new DictionaryRailingStyle(m_db);
            if (!dict.Has("LandingExtension", m_compBldr.Transaction)) {
                ArchDb.RailingStyle oldRailingStyle = m_compBldr.Transaction.GetObject(railing.StyleId, OpenMode.ForRead) as RailingStyle;
                ArchDb.RailingStyle newRailingStyle = new RailingStyle();
                newRailingStyle.SetToStandard(m_db);
                newRailingStyle.SubSetDatabaseDefaults(m_db);
                newRailingStyle.UseStairLandingExtensionAtLanding = false;
                dict.AddNewRecord("LandingExtension", newRailingStyle);
                m_compBldr.Transaction.AddNewlyCreatedDBObject(newRailingStyle, true);
            }

            railing.StyleId = dict.GetAt("LandingExtension");

            /// TBD: extend the railing to the end of the landing
            //Autodesk.Aec.Geometry.Polyline2d path = new Autodesk.Aec.Geometry.Polyline2d();
            //path.AddSegment(new Autodesk.Aec.Geometry.Segment2d(startPt, endPt), true);
            //railing.SetPath(path);

            // get an anchor to attach to
            AnchorRailingToStair anchor = new AnchorRailingToStair();
            anchor.SetToStandard(m_db);
            anchor.StairId = stairId;

            if (isCounterClockwise)
                anchor.Justification = StairJustificationType.Right;
            else
                anchor.Justification = StairJustificationType.Left;

            /// note that a component index 0f -1 means that the 
            /// railing is anchored to the whole stair
            anchor.ComponentIndex = 2;

            railing.SetAnchor(anchor);
            m_compBldr.AddToDb(railing);

            return railing.Id;
        }

        private AcDb.ObjectIdCollection
        CreateExteriorWindows (AcDb.ObjectId wallId)
        {
            Transaction tr = m_compBldr.Transaction;
            ArchDb.Wall wall = tr.GetObject(wallId, OpenMode.ForRead) as ArchDb.Wall;

            AcDb.ObjectIdCollection extWindowIds = new AcDb.ObjectIdCollection();

            // create a row of windows in the front
            double spacing = 24.0;

            double winWidth = 60.0;
            double winHeight = 48.0;

            int rem = 0;
            int count = Math.DivRem(Convert.ToInt32(wall.Length), Convert.ToInt32(winWidth + spacing), out rem);
            double extaOffset = rem / count;

            double xDist = (spacing / 2) + extaOffset;
            double yDist = spacing;     // start 24" down from top of wall

            for (int i = 0; i < 3; i++) {
                for (int j = 0; j < count; j++) {
                    Window tmpWin = new Window();
                    tmpWin.Width = winWidth;
                    tmpWin.Height = winHeight;
                    m_compBldr.AddToDb(tmpWin);

                    AnchorOpeningBaseToWall anchor1 = new AnchorOpeningBaseToWall();
                    anchor1.CurveId = wallId;
                    anchor1.AnchorX.MeasureToType = CurveXMeasureToType.ToStartEdge;
                    anchor1.AnchorX.OffsetType = CurveXOffsetType.FromStartOfCurve;
                    anchor1.AnchorX.OffsetDistance = xDist;

                    anchor1.AnchorY.OffsetType = CurveYOffsetType.FromRightEdge;
                    anchor1.AnchorY.MeasureToType = CurveYMeasureToType.ToFrontEdge;
                    anchor1.AnchorY.OffsetDistance = 0.0;

                    anchor1.AnchorZ.OffsetFromType = CurveZOffsetFromType.FromTop;
                    anchor1.AnchorZ.OffsetToType = CurveZOffsetToType.ToTop;
                    anchor1.AnchorZ.OffsetDistance = -yDist;      // make the windows 24" from top of window to top of wall

                    tmpWin.SetAnchor(anchor1);
                    extWindowIds.Add(tmpWin.ObjectId);

                    xDist += winWidth + spacing + extaOffset;
                }
                xDist = (spacing / 2) + extaOffset;     // reset xDist
                yDist += 120.0;   // move down for each floor
            }

            return extWindowIds;
        }

        private Point3dCollection
        ConvertTo3d (Point2dCollection pts2d, double zOffset)
        {
            Point3dCollection pts = new Point3dCollection();
            foreach (Point2d pt in pts2d) {
                pts.Add(new Point3d(pt.X, pt.Y, zOffset));
            }
            return pts;
        }


        private Point2dCollection
        Building1_OutlinePoints ()
        {
            Point2dCollection pts = new Point2dCollection();

            double x = 0.0;
            double y = 0.0;

            pts.Add(new Point2d(x, y));

            x += 200.0;
            pts.Add(new Point2d(x, y));

            y += 200.0;
            pts.Add(new Point2d(x, y));

            x += 400.0;
            pts.Add(new Point2d(x, y));

            y -= 200.0;
            pts.Add(new Point2d(x, y));

            x += 200.0;
            pts.Add(new Point2d(x, y));

            y += 400.0;
            pts.Add(new Point2d(x, y));

            x -= 800.0;
            pts.Add(new Point2d(x, y));

            y -= 400.0;
            pts.Add(new Point2d(x, y));

            return pts;
        }

        private Point2dCollection
        SlabHolePoints (double xOffset)
        {
            Point2dCollection pts = new Point2dCollection();

            double x = xOffset;
            double y = 400.0;

            pts.Add(new Point2d(x, y));

            x += 150.0;
            pts.Add(new Point2d(x, y));

            y -= 132.0;
            pts.Add(new Point2d(x, y));

            x -= 150.0;
            pts.Add(new Point2d(x, y));

            y += 132.0;
            pts.Add(new Point2d(x, y));

            return pts;
        }
    }
}
